import {
  Loader, Metadata, IEventParam, Http, IDictionary, Datasource,
      
} from '@zeedhi/core';
    
import {
  GridEditable, Select, Form, TextInput, IComponentRender, Card, Button,
} from '@zeedhi/common';
    
export class Pla00470EmbalagemController {
  private transpCardVisible = false;
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public newRow = false;
  
  private grid!: GridEditable;
  
  public crudController = Loader.getInstance('CrudController');
  
  public utilities = Loader.getInstance('Utilities');
  
  private cancelActionsAfterDrag(gridEmbalagem: GridEditable) {
    const toolbarSlot = gridEmbalagem.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = true;
      if (e.name === 'saveButton') e.disabled = true;
    });
  }
  
  public onChangeEmbalagem({ component }: IEventParam<Datasource>) {
    const colunaEmb = component.currentRow.CDCONTAINER;
    
    if (colunaEmb) {
      const gridEmbalagem = Metadata.getInstance<GridEditable>('relacaoTranspGridDetail');
      
      const conditions = [
        {
          name: 'CDCONTAINER',
          operator: '=',
          value: component.currentRow.CDCONTAINER,
        },
      ];
      
      gridEmbalagem.datasource.filter = conditions;
      gridEmbalagem.datasource.get();
      this.cardEmbalagemVisible();
    }
  }
  
  private cardEmbalagemVisible() {
    const card = Metadata.getInstance<Card>('relacaoTranspCard');
    try {
      if (!this.gridEditController.newRow) {
        card.isVisible = true;
        this.transpCardVisible = true;
      }
    } catch (Error) {
      card.isVisible = false;
      this.transpCardVisible = false;
    }
  }
  
  public closeCardRelTransp() {
    const card = Metadata.getInstance<Card>('relacaoTranspCard');
    
    card.isVisible = false;
    this.transpCardVisible = false;
  }
  
  public toUpperCase({ component }: IEventParam<TextInput>) {
    component.value = component.value.toUpperCase();
  }
  
  get gridColsReturnValue(): number {
    return this.transpCardVisible ? 6 : 12;
    this.reload();
  }

  public cancelEditing({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    if (grid.editing) this.defaultState(grid);
  }
  
  public defaultState(grid: GridEditable) {
    this.newRow = false;
    return grid.cancelEditedRows().then(() => {
      this.gridEditController.onInlineEdit({ component: grid });
      return grid.datasource.get();
    });
  }
  
  public reload() {
    const grid = Metadata.getInstance<GridEditable>('gridEmbalagem');
    this.gridEditController.reload({ component: grid });
  }
  
  public saveContainer() {
    const form = Metadata.getInstance<Form>('formEmbalagem');
    const gridEmbalagem: GridEditable = Metadata.getInstance('gridEmbalagem');
    const editedRows: IDictionary = gridEmbalagem.getEditedRows();
    
    if (form.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        if (row.__is_new) {
          row.CDCONTAINER = row.CDCONTAINER.padStart(5, '0');
          
          if (row.IDEMBALAGEM === 1 && row.IDPROPOSITO === 'I') {
            row.IDPROPOSITO = 'C';
          }
          
          row.NRCATEMBALAG = row.NRCATEMBALAG ? row.NRCATEMBALAG : null;
          row.SGUNIDADE = row.SGUNIDADE ? row.SGUNIDADE : null;
          row.VRPESOCONTAINER = row.VRPESOCONTAINER ? row.VRPESOCONTAINER : null;
        } else row.__is_new = false;
      });
      
      this.crudController.saveRows(editedRows, 'container', gridEmbalagem).then(() => {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        this.reload();
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public setPurpose() {
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const embalagem = grid.getChildInstance<Select>('IDEMBALAGEM_editable_');
    const proposito = grid.getChildInstance<Select>('IDPROPOSITO_editable_');
    
    if (embalagem.datasource.currentRow.value === 1) proposito.setValue('C');
  }
  
  public changePurpose() {
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const embalagem = grid.getChildInstance<Select>('IDEMBALAGEM_editable_');
    const proposito = grid.getChildInstance<Select>('IDPROPOSITO_editable_');
    
    if (embalagem.datasource.currentRow.value === 1 && proposito.datasource.currentRow.value !== 'C') {
      proposito.setValue('');
      this.utilities.showAlert('alert', 'pme::transportadaPropColetivo', 'warning');
    }
  }
  
  public saveEmbalagem() {
    const form = Metadata.getInstance<Form>('formRelacaoTransp');
    const grid: GridEditable = Metadata.getInstance('relacaoTranspGridDetail');
    const editedRows: IDictionary = grid.getEditedRows();
    const gridEmbalagem: GridEditable = Metadata.getInstance('gridEmbalagem');
    
    if (editedRows.length > 0) {
      Http.post('/retornaCodigoEmbalagem', {}).then((response: IDictionary) => {
        const codigo = response.data.dataset.codigo[0].CODIGO;
        
        editedRows.forEach((row: IDictionary) => {
          if (row.__is_new) {
            row.NRCONTEMBALAG = codigo;
            row.CDCONTAINER = gridEmbalagem.datasource.currentRow.CDCONTAINER;
            row.CDEMBALAGEM = row.NMCONTAINER ? row.NMCONTAINER.CDCONTAINER : null;
            row.NMCONTAINER = row.NMCONTAINER ? row.NMCONTAINER.NMCONTAINER : null;
            row.QTCAPACIDADE = row.QTCAPACIDADE ? row.QTCAPACIDADE : null;
            if (row.CDCONTAINER === row.CDEMBALAGEM) {
              this.utilities.showAlert('alert', 'pme::relacaoEmbalagemIguais', 'warning');
              const embalagem = grid.getChildInstance<Select>('NMCONTAINER_editable_');
              embalagem.setValue('');
            }
          } else row.__is_new = false;
          
          if (form.validate()) {
            this.crudController.saveRows(editedRows, 'contembalag', gridEmbalagem).then(() => {
              this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
              this.cancelButton();
            }).catch((error: string) => {
              this.utilities.showAlert('alert', `pme::${error}`, 'error');
            });
          }
        });
      });
    }
  }
  
  public addRowRel() {
    const grid: GridEditable = Metadata.getInstance('relacaoTranspGridDetail');
    const buttonSave = Metadata.getInstance<Button>('saveButtonRel');
    const buttonCancel = Metadata.getInstance<Button>('cancelButtonRel');
    this.gridEditController.addRow({ component: grid });
    buttonSave.disabled = false;
    buttonCancel.disabled = false;
  }
  
  public cancelButton() {
    const grid: GridEditable = Metadata.getInstance('relacaoTranspGridDetail');
    const buttonSave = Metadata.getInstance<Button>('saveButtonRel');
    const buttonCancel = Metadata.getInstance<Button>('cancelButtonRel');
    this.gridEditController.cancelEditing({ component: grid });
    buttonSave.disabled = true;
    buttonCancel.disabled = true;
  }
}
