import {
  ModalService, Modal, Form, Select, Grid, IButtonEvent, GridEditable, IComponentRender,
} from '@zeedhi/common';
import {
  Loader, Metadata, IEventParam, IDictionary,
} from '@zeedhi/core';

export class Pla01309RevisaoAutomaticaController {
  private utilities = Loader.getInstance('Utilities');
  
  public modalRevisaoAutoReceitas!: Modal;
  
  private async createModalRevisaoAutoReceitas() {
    this.modalRevisaoAutoReceitas = await ModalService.createFromJson('planejamento/prato/modals/pla01309_revisao_auto_receitas', true);
  }
    
  public closeModalRevisaoAutoReceitas() {
    this.modalRevisaoAutoReceitas.hide();
  }
    
  public openModalRevisaoAutoReceitas() {
    if (!this.modalRevisaoAutoReceitas) {
      this.createModalRevisaoAutoReceitas().then(() => {
        this.modalRevisaoAutoReceitas.show();
      });
    } else {
      this.modalRevisaoAutoReceitas.show();
    }
  }
  
  public fullscreen({ component }: IButtonEvent) {
    this.modalRevisaoAutoReceitas.fullscreen = !this.modalRevisaoAutoReceitas.fullscreen;
    component.iconName = (this.modalRevisaoAutoReceitas.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
  
  public inicContainerDestino() {
    const project = localStorage.getItem('PRODUCT_ID');
    
    const tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    
    const tpReceitaDestCard = Metadata.getInstance<Select>('TPRECEITADEST');
    const tpReceitaDest = Metadata.getInstance<Select>('TPRECEITADEST1');

    const cardapio = Metadata.getInstance<Select>('TIPOCARD');
    const cardapioDest = Metadata.getInstance<Select>('TIPOCARDDEST');

    if (project === '4875') {
      tpReceitaCard.isVisible = true;
      tpReceita.isVisible = false;
      tpReceitaDestCard.isVisible = true;
      tpReceitaDest.isVisible = false;
    } else {
      tpReceitaCard.isVisible = false;
      tpReceita.isVisible = true;
      tpReceitaDestCard.isVisible = false;
      tpReceitaDest.isVisible = true;
      cardapio.isVisible = false;
      cardapioDest.isVisible = false;
    }
  }
  
  public setFilterServiceDest({ component }: IEventParam<Select>) {
    const form: Form = Metadata.getInstance('formFiltroRevAuto');
    const row = form.value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: row.UNIDADEDEST,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: localStorage.getItem('PRODUCT_ID') === '4875' ? '' : 'N',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public changeRecipeRevisaoAuto({ component }: IEventParam<Select>) {
    const filial = Metadata.getInstance<Select>('UNIDADEDEST');
    const cardapio = Metadata.getInstance<Select>('TIPOCARDDEST');
    const servico = Metadata.getInstance<Select>('SERVICODEST');

    if (component.value === 'L') {
      filial.disabled = false;
      filial.addValidation('required', { message: 'pme::campoObrigatorio' });
      filial.setValue('');
      servico.disabled = true;
      servico.removeValidation('required');
      servico.setValue('');
      cardapio.disabled = true;
      cardapio.removeValidation('required');
      cardapio.setValue('');
    } else if (component.value === 'T') {
      filial.disabled = true;
      filial.removeValidation('required');
      filial.setValue('');
      servico.disabled = true;
      servico.removeValidation('required');
      servico.setValue('');
      cardapio.addValidation('required', { message: 'pme::campoObrigatorio' });
      cardapio.disabled = false;
    } else if (component.value === 'S') {
      filial.disabled = false;
      servico.disabled = false;
      cardapio.disabled = true;
      filial.addValidation('required', { message: 'pme::campoObrigatorio' });
      servico.addValidation('required', { message: 'pme::campoObrigatorio' });
      cardapio.removeValidation('required');
      cardapio.setValue('');
    } else if (component.value === 'P') {
      // fazer verificacao pro opLogado
      filial.removeValidation('required');
      servico.removeValidation('required');
      cardapio.removeValidation('required');
      filial.disabled = true;
      servico.disabled = true;
      cardapio.disabled = true;
      filial.setValue('');
      servico.setValue('');
      cardapio.setValue('');
    }
  }
  
  public saveRevisaoAutomatica() {
    const form = Metadata.getInstance<Form>('formAcoesPrato');
    const grid: GridEditable = Metadata.getInstance('gridPrato');
    const editedRows = grid.getEditedRows() as IDictionary;

    if (form.validate() && editedRows.length > 0) {
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      this.atualizarGridRevisaoAutomatica();
    }
  }
    
  public atualizarGridRevisaoAutomatica() {
    const grid: GridEditable = Metadata.getInstance('gridPrato');
    const editedRows = grid.getEditedRows() as IDictionary;
    const newPorc = editedRows[0].QUANTIDADE;
    grid.datasource.currentRow.QUANTIDADE = newPorc;
    grid.editing = false;
    const toolbarSlot = grid.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = !e.disabled;
      if (e.name === 'saveButton') e.disabled = !e.disabled;
    });
  }
  
  public confirmarAcoesRevisaoAutomatica() {
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const row = form.value;
    const form2: Form = Metadata.getInstance('formFiltroRevAuto');
    const row2 = form2.value;
    const grid: Grid = Metadata.getInstance('gridPrato');
    const rowsGrid = grid.selectedRows;
    
    if (row.TPRECEITA === null) row.TPRECEITA = row.TPRECEITA1;
    if (row2.TPRECEITADEST === null) row2.TPRECEITADEST = row2.TPRECEITADEST1;
    
    if (row.TPRECEITA === 'P' && row2.TPRECEITADEST === 'P') this.utilities.showAlert('Alert', 'pme::pratoPadraoOrigemDestino', 'error');
    if (form.validate() && form2.validate()) {
      if (row.TPRECEITA === 'L' && row2.TPRECEITADEST === 'L' && row.UNIDADE === row2.UNIDADEDEST) this.utilities.showAlert('Alert', 'pme::bloqueiaUnidadeIgual', 'error');
      else if (rowsGrid.length > 0) {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        this.closeModalRevisaoAutoReceitas();
      } else this.utilities.showAlert('Alert', 'pme::aoMenosUmPratoSelecionado', 'error');
    }
  }
}
