import {
  Select, Modal, ModalService, Radio, Button, Form, Number, SelectMultiple,
} from '@zeedhi/common';
import {
  Metadata, IEventParam, Loader, Http, I18n,
} from '@zeedhi/core';

export class QuantidadePorPessoaController {
  private appController = Loader.getInstance('AppController');

  private utilities = Loader.getInstance('Utilities');
  
  private pratoController = Loader.getInstance('Pla01300PratoController');
  
  public modalCalcQuantidadePessoa!: Modal;
  
  public inicContainer() {
    const project = localStorage.getItem('PRODUCT_ID');
    const cardapio = Metadata.getInstance<Select>('TPCARD');

    if (project === '4875') cardapio.isVisible = true;
    else cardapio.isVisible = false;
  }

  private async createModalCalcQuantidadePessoa() {
    this.modalCalcQuantidadePessoa = await ModalService.createFromJson('planejamento/prato/modals/formCalcQuantidadePessoa', true);
  }

  public closeModalCalcQuantidadePessoa() {
    this.modalCalcQuantidadePessoa.hide();
  }
  
  public openModalCalcQuantidadePessoa() {
    if (!this.modalCalcQuantidadePessoa) {
      this.createModalCalcQuantidadePessoa().then(() => {
        this.modalCalcQuantidadePessoa.show();
      });
    } else {
      this.modalCalcQuantidadePessoa.show();
    }
  }
  
  public tratProduct() {
    const tipoCard: SelectMultiple = Metadata.getInstance<SelectMultiple>('TPCARD');
    const receita = Metadata.getInstance<SelectMultiple>('IDTIPORECEITA');
    
    if (this.pratoController.project === 'PAE') {
      tipoCard.isVisible = true;
      // const newData = receita.datasource.data.filter((item) => item.value !== 'T');
      receita.datasource.updateData([
        {
          name: I18n.translate('pme::padrao'),
          value: 'P',
        },
        {
          name: I18n.translate('pme::tipoDeCardapio'),
          value: 'T',
        },
        {
          name: I18n.translate('pme::local'),
          value: 'L',
        },
        {
          name: I18n.translate('pme::servico'),
          value: 'S',
        },
      ]);
    } else {
      tipoCard.isVisible = false;
      receita.datasource.updateData([
        {
          name: I18n.translate('pme::padrao'),
          value: 'P',
        },
        {
          name: I18n.translate('pme::local'),
          value: 'L',
        },
        {
          name: I18n.translate('pme::servico'),
          value: 'S',
        },
      ]);
    }
  }
  
  // validar
  public setFilterTpCardapio() {
    const cardapio = Metadata.getInstance<Select>('TPCARD');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    cardapio.datasource.filter = filter;
    cardapio.datasource.get();
  }
  
  public radioConfig({ component }: IEventParam<Radio>) {
    const valorFixo = Metadata.getInstance<Number>('VRQTDEPESSOA');
    const valorRadio = component.value;
    if (valorRadio === '0') {
      valorFixo.addValidation('required', { message: 'pme::campoObrigatorio' });
      valorFixo.disabled = false;
    } else {
      valorFixo.removeValidation('required');
      valorFixo.disabled = true;
    }
  }
  
  public procedureQtdPessoa({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formFilter');
    
    const arrReceitas = form.value.IDTIPORECEITA;
    const findRecPadrao = arrReceitas.find((column: any) => column === 'P');
    
    if (form.validate()) {
      const valorFixo = Metadata.getInstance<Number>('VRQTDEPESSOA');
      const opcao = Metadata.getInstance<Radio>('ORDENA');
      
      if ((opcao.value === '0' && valorFixo.value > '0') || opcao.value === '1') {
        const row = form.value;
        row.CDOPERADOR = localStorage.getItem('CDOPERADOR');
        row.CDFILCENTPME = this.appController.getCdCentral();
        row.IDTIPOSERV = this.pratoController.project === 'PAE' ? 'M' : 'N';
        
        if (findRecPadrao === 'P') {
          if (this.pratoController.operatorValid !== false) {
            this.utilities.activeLoadingButton(component);
            Http.post('/procedureQtdPessoa', { requestType: 'Row', row }).then(() => {
              this.utilities.showAlert('alert', 'pme::atualizadoComSucesso', 'success');
            }).finally(() => {
              this.utilities.disableLoadingButton(component);
            });
          } else {
            this.utilities.showAlert('Alert', 'pme::modificacaoRecPadrao', 'warning');
            this.pratoController.openModalGroupOperador(component);
          }
        } else {
          this.utilities.activeLoadingButton(component);
          Http.post('/procedureQtdPessoa', { requestType: 'Row', row }).then(() => {
            this.utilities.showAlert('alert', 'pme::atualizadoComSucesso', 'success');
          }).finally(() => {
            this.utilities.disableLoadingButton(component);
          });
        }
      } else {
        this.utilities.showAlert('error', 'pme::qtdMaiorZero', 'red');
        this.utilities.disableLoadingButton(component);
      }
    }
  }
  
  public setFilterAgregAgrup({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    
    const conditions = [
      {
        name: 'P_CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value?.CDGRUPFILI ? agrupUnid.value.CDGRUPFILI : 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value.CDGRUPFILI : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPOSERVICO');
    const grupServValue = grupserv.value ? grupserv.value : 'T';
    
    const filter = [
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: localStorage.getItem('PRODUCT_ID') === '4875' ? '' : 'N',
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public controllPratoIni() {
    const pratoFinal = Metadata.getInstance<Select>('PRATO_FINAL');
    if (pratoFinal.value !== null) pratoFinal.setValue(null);
  }
  
  public verifyPratIni({ component }: IEventParam<Select>) {
    if (component.value !== null) {
      const pratoIni = Metadata.getInstance<Select>('PRATO_INICIAL');
      if (this.utilities.returnPrat(pratoIni.value, component.value)) {
        this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
        component.setValue(null);
      }
    }
  }
}
